<?php

declare(strict_types=1);

namespace Drupal\konfhub;

use Drupal\Component\Datetime\TimeInterface;
use Drupal\Component\Utility\Random;
use Drupal\Core\Database\Connection;
use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\user\Entity\User;
use Drupal\user\UserInterface;
use Drupal\user\UserStorageInterface;
use Psr\Log\LoggerInterface;

/**
 * Konfhub Ticket Details Manager.
 */
final class TicketDetailsManager {

  private UserStorageInterface|EntityStorageInterface $userStorage;

  /**
   * Constructs a Manager object.
   */
  public function __construct(
    protected Connection $connection,
    protected LoggerInterface $logger,
    protected TimeInterface $time,
    EntityTypeManagerInterface $entityTypeManager,
  ) {
    $this->userStorage = $entityTypeManager->getStorage('user');
  }

  /**
   *
   */
  public function processData(array $payload): bool {
    $type = $payload['Event Type'];
    $data = $payload['Data'];

    return match ($type) {
      'registration', 'cancel' => $this->handleRegistration($type, $data['Attendee Details']),
      'lead' => $this->handleLeadGeneration($type, $data['Attendee Details']),
      'check_in', 'check_out' => $this->handleCheckDetails($type, $data[$type === 'check_in' ? 'Check In Updates' : 'Check Out Updates']),
      default => FALSE,
    };
  }

  /**
   *
   */
  private function checkRegistrationDetails(string $bookingId, int $userId): bool|array {
    $query = $this->connection->select('konfhub_ticket_details', 'ktd')
      ->fields('ktd')
      ->condition('booking_id', $bookingId)
      ->condition('user_id', $userId)
      ->execute();

    return $query->fetchAssoc() ?: FALSE;
  }

  /**
   *
   */
  private function createRegistration(array $details): int {
    $fields = [
      'type' => $details['type'],
      'booking_id' => $details['booking_id'],
      'user_id' => $details['user_id'],
      'details' => json_encode($details['details']),
      'created' => $this->time->getRequestTime(),
      'changed' => $this->time->getRequestTime(),
    ];

    $id = $this->connection->insert('konfhub_ticket_details')
      ->fields($fields)
      ->execute();

    $this->logger->notice('Inserted ticket with ID %id', ['%id' => $id]);

    return (int) $id;
  }

  /**
   *
   */
  private function updateRegistration(int $id, array $details): int {
    $fields = [
      'type' => $details['type'],
      'booking_id' => $details['booking_id'],
      'user_id' => $details['user_id'],
      'details' => json_encode($details['details']),
      'changed' => $this->time->getRequestTime(),
    ];

    $affectedRows = $this->connection->update('konfhub_ticket_details')
      ->fields($fields)
      ->condition('id', $id)
      ->execute();

    $this->logger->notice('Updated ticket with ID %id', ['%id' => $id]);

    return (int) $affectedRows;
  }

  /**
   *
   */
  private function handleRegistration(string $type, array $attendee): bool {
    $user = $this->createOrGetUser($attendee['Email Address']);
    $bookingId = $attendee['Booking Id'];
    $existingDetails = $this->checkRegistrationDetails($bookingId, (int) $user->id());

    $data = [
      'type' => $type,
      'booking_id' => $bookingId,
      'user_id' => $user->id(),
      'details' => [
        'name' => $attendee['Name'],
        'bookingId' => $bookingId,
        'paymentId' => $attendee['payment_id'],
        'paymentDetails' => $attendee['Amount Details'],
      ],
    ];

    if (!$existingDetails) {
      $this->createRegistration($data);
      return TRUE;
    }

    if ($existingDetails['type'] !== $type) {
      $this->updateRegistration((int) $existingDetails['id'], $data);
      return TRUE;
    }

    return FALSE;
  }

  /**
   *
   */
  private function handleCheckDetails(string $type, array $attendee): bool {
    $user = $this->createOrGetUser($attendee['Email Address']);
    $bookingId = $attendee['Booking Id'];

    $query = $this->connection->select('konfhub_check_details', 'kcd')
      ->fields('kcd', ['id'])
      ->condition('booking_id', $bookingId)
      ->condition('user_id', $user->id())
      ->condition('type', $type)
      ->execute();

    if ($query->fetchField()) {
      return FALSE;
    }

    $dateTime = new \DateTime($attendee['CheckIn Time']);
    $data = [
      'type' => $type,
      'booking_id' => $bookingId,
      'user_id' => $user->id(),
      'check_at' => $dateTime->getTimestamp(),
    ];

    $this->connection->insert('konfhub_check_details')
      ->fields($data)
      ->execute();

    return TRUE;
  }

  /**
   *
   */
  private function handleLeadGeneration(string $type, array $attendee): bool {
    $user = $this->createOrGetUser($attendee['Email Address']);
    $bookingId = $attendee['Booking Id'];

    $query = $this->connection->select('konfhub_ticket_details', 'ktd')
      ->fields('ktd', ['id'])
      ->condition('booking_id', $bookingId)
      ->condition('user_id', $user->id())
      ->condition('type', $type)
      ->execute();

    if ($query->fetchField()) {
      return FALSE;
    }

    $data = [
      'type' => $type,
      'booking_id' => $bookingId,
      'user_id' => $user->id(),
    ];

    $this->createRegistration($data);

    return TRUE;
  }

  /**
   *
   */
  private function createOrGetUser(string $email): UserInterface {
    $users = $this->userStorage->loadByProperties(['mail' => trim($email)]);
    $user = reset($users);

    if (!$user instanceof UserInterface) {
      $random = new Random();
      $user = User::create([
        'mail' => $email,
        'name' => $email,
        'pass' => $random->string(),
      // Blocked.
        'status' => 0,
      ]);
      $user->enforceIsNew();
      $user->save();
    }

    return $user;
  }

}
